#include "ell/MinimumSelector.hh"

namespace ell
{
	
	////////////////////////////////////////////////////////////////////////////
	MinimumSelector::~MinimumSelector()
	////////////////////////////////////////////////////////////////////////////
	{
	}

} // namespace ell

#include "biu/RandomNumberFactory.hh"

namespace ell
{
	////////////////////////////////////////////////////////////////////////////
	MS_Uniform::
	MS_Uniform()
	////////////////////////////////////////////////////////////////////////////
	{
	}
	
	
	////////////////////////////////////////////////////////////////////////////
	MS_Uniform::
	~MS_Uniform()
	////////////////////////////////////////////////////////////////////////////
	{
	}

	////////////////////////////////////////////////////////////////////////////
	size_t
	MS_Uniform::
	select(	const LandscapeTopology* const lt
			, const bool minimaChanged )
	////////////////////////////////////////////////////////////////////////////
	{
		size_t selected = LandscapeTopology::INVALID_INDEX;
		
		  // select uniformly a minimum 
		if (lt->getMinCount() > 0) {
			selected = (size_t)biu::RNF::getRN((unsigned int)lt->getMinCount());
		}	
		
		return selected;
	}
	
} // namespace ell

#include <cmath>
#include <algorithm>

namespace ell
{
	////////////////////////////////////////////////////////////////////////////
	MS_Boltzmann::
	MS_Boltzmann( const double shiftFactor_ )
	  :	shiftFactor( shiftFactor_ )
	  	, boltzWeightSum()
	////////////////////////////////////////////////////////////////////////////
	{
	}
	
	
	////////////////////////////////////////////////////////////////////////////
	MS_Boltzmann::
	~MS_Boltzmann()
	////////////////////////////////////////////////////////////////////////////
	{
	}

	////////////////////////////////////////////////////////////////////////////
	size_t
	MS_Boltzmann::
	select(	const LandscapeTopology* const lt
			, const bool minimaChanged )
	////////////////////////////////////////////////////////////////////////////
	{
		size_t selected = LandscapeTopology::INVALID_INDEX;
		
		  // select uniformly a minimum 
		if (lt->getMinCount() > 0) {
			  // check if boltzWeightSum has to be updated
			if (minimaChanged || boltzWeightSum.size()!=(lt->getMinCount()+1)) {
				  // resize the weight sum container
				boltzWeightSum.resize(lt->getMinCount()+1,0.0);
				  // sum successively over the Boltzmann weights
				boltzWeightSum[0] = 0.0;
				for (size_t i=1; i<boltzWeightSum.size(); i++) {
					boltzWeightSum[i] = exp( -lt->getMin(i-1)->getEnergy()
											/ LandscapeTopology::BOLTZMANN_KT )
										+ boltzWeightSum[i-1];
				}
			}
			  // calculate a random number in interval [0,fullBoltzWeightSum]
			double selectedProp = (((double) biu::RNF::getRN()) 
										/ ((double) biu::RNF::getMaxRN())) 
									* (*(boltzWeightSum.rbegin()));
			  // get index of the corresponding interval to find the minimum
			std::vector<double>::const_iterator pos
				= std::lower_bound(	boltzWeightSum.begin()
									, boltzWeightSum.end()
									, selectedProp );
			  // recalculate position from iterators
			selected = (size_t)(pos - boltzWeightSum.begin());
		}
		
		return selected;
	}
	
} // namespace ell


