#include "ell/StateCollector.hh"

namespace ell {
	
	SC_Listing::SC_Listing()
	  :	stateList() 
	{}
	
	SC_Listing::~SC_Listing()
	{
		  // delete all collected State objects
		for (	List::iterator it = stateList.begin();
				it != stateList.end(); it++ ) 
		{
			delete *it;
		}
		  // clear NULL pointer list
		stateList.clear();
	}

	  // This function is used to track all added intermediate States.
	  // @param s the added State
	void 
	SC_Listing::add(const State& s) {
		  // store a copy of s
		stateList.push_back(s.clone());
	}
	
	  // Returns number of added States.
	  // @return the number of added states
	size_t 
	SC_Listing::size() const {
		return stateList.size();
	}
	 
	  // Returns last added State.
	  // @return the last added State
	const State* const 
	SC_Listing::getLastAdded() const {
		return *(stateList.rbegin());
	}
	
	///////////  LIST HANDLING  ////////////////////////

	  // Returns a list of all added States.
	  // @return the State list
	const SC_Listing::List&
	SC_Listing::getList() const {
		return stateList;
	}
	

} // namespace ell

namespace ell {
	
	SC_ListingK::SC_ListingK(const size_t length)
	  :	stateList(length,NULL)
	  	, curEnd(0)
	  	, addedTotal(0)
	{
		assertbiu(length > 0, "the minimal list length should be 1");
	}
	
	SC_ListingK::~SC_ListingK()
	{
		  // delete all collected State objects
		for (	List::iterator it = stateList.begin();
				it != stateList.end(); it++ ) 
		{
			delete *it;
		}
		  // clear NULL pointer list
		stateList.clear();
	}

	  // This function is used to track all added intermediate States.
	  // @param s the added State
	void 
	SC_ListingK::add(const State& s) {
		  // count add
		addedTotal++;
		
		  // get index of next position to overwrite
		curEnd = (curEnd+1) % stateList.size();
		
		  // store a copy of s
		stateList[curEnd] = s.clone(stateList[curEnd]);
	}
	
	  // Returns number of added States.
	  // @return the number of added states
	size_t 
	SC_ListingK::size() const {
		return addedTotal;
	}
	 
	  // Returns last added State.
	  // @return the last added State
	const State* const 
	SC_ListingK::getLastAdded() const {
		return stateList[curEnd];
	}

	SC_ListingK::iterator 
	SC_ListingK::getList() const {
		return iterator( (curEnd+1)%stateList.size(), this);
	}

} // namespace ell
