#ifndef S_LP_PIVOTM_HH_
#define S_LP_PIVOTM_HH_

#include "ell/protein/S_LP.hh"
#include <biu/LatticeModel.hh>
#include <biu/LatticeProtein_I.hh>
#include <biu/LatticeMoveSet.hh>
#include <string>

namespace ell
{

class S_LP_PivotM : public ell::S_LP
{
	
public:

	  //! the specific ID string of this class
	static const std::string ID;

	  //! Constructs a State using a LatticeProtein and a PullMoveSet.
	  //! @param latProt 	the lattice protein used by this State
	  //! @param moveSet	the move set used by this State	
	S_LP_PivotM(	biu::LatticeProtein_I* latProt,
				biu::LatticeMoveSet* moveSet);
	
	S_LP_PivotM( const S_LP_PivotM&);
	
	virtual ~S_LP_PivotM();

		/*!
		 * Access to a State subclass specific ID string to identify 
		 * instances of this class.
		 * @return the subclass specific ID string
		 */
	virtual const std::string& getID( void ) const;

	/*! Returns a pointer to a clone of the current state. 
	 * @param toFill a State to make a copy of this, or NULL if a new 
	 *        State should be created
	 * @return a pointer to a copy of this state, if toFill is != NULL
	 *         the return value corresponds to the updated state pointed
	 *         to by toFill, otherwise a new object is created !!!
	 * */
	virtual S_LP_PivotM* clone(State* toFill = NULL) const;
	
	  /*! Returns a new State based on the current state. The new state
	   *  differs only by the information given by stringRep. */
	virtual State* fromString(const std::string& stringRep) const;		
	
	//////////////  COMPRESSED REPRESENTATION  ////////////////////////////
	
		/*! Access to a compressed sequence representation of the state.
		 * @return the compressed sequence representation
		 */
	virtual CSequence compress(void) const;
	
		/*! Access to a compressed sequence representation of the state.
		 * @param toFill a data structure to write the compressed 
		 *               representation too
		 * @return the compressed sequence representation
		 */
	virtual CSequence& compress(CSequence& toFill) const;
	
		/*! Uncompresses a compressed sequencce representation into a new
		 * State object.
		 * @param cseq the compressed sequence representation of a state
		 * @param toFill a state object to uncompress too or NULL if a new 
		 *               object has to be created
		 * @return new State object that is encoded in cseq or NULL in error
		 *         case.
		 */
	virtual State* uncompress(const CSequence& cseq, State* toFill ) const;
	
		/*! Uncompresses a compressed sequencce representation into a this
		 * State object.
		 * @param cseq the compressed sequence representation of a state
		 * @return this or NULL in error case
		 */
	virtual State* uncompress(const CSequence& cseq);
	
	virtual unsigned int getMinimalDistance(const State& state2) const { 
		return UINT_MAX; 
	};
};

} // end namespace ell
#endif /*S_LP_PIVOTM_HH_*/
