#include <iostream>
#include <iomanip>

#include <biu/RandomNumberGenerator.hh>
#include <biu/RandomNumberFactory.hh>

#include "ell/S_Explicit.hh"
#include "ell/Walk.hh"
#include "ell/BarrierSampling.hh"
#include "ell/LT_BarrierTree.hh"



int main( int argc, char** argv ) {

	using namespace ell;
	

	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test function : BS_UpDown\n"
				<<"===========================================" <<std::endl;

		// init randomizer	
	std::cout	<<"=> RND::setRNG( biu::RNG_LCG() )" <<std::endl;
	biu::RandomNumberGenerator* rng = new biu::RNG_LCG();
	biu::RNF::setRNG( rng );
	delete rng;
	const unsigned int seed = 1;
	std::cout	<<"=> biu::RNF::setSeed("<<seed<<")" <<std::endl;
	biu::RNF::getRNG().setSeed(seed);
	
	
	std::cout	<< "=> creating S_Explicit::LandscapeGraph lg" << std::endl;
	S_Explicit::LandscapeGraph lg;
	
	lg.addNode( 3 ); // 0
	lg.addNode( 5 ); // 1
	lg.addNode( 1 ); // 2
	lg.addNode( 4 ); // 3
	lg.addNode( 7 ); // 4
	lg.addNode( 6 ); // 5
	lg.addNode( 2 ); // 6
	lg.addNode( 7 ); // 7
	lg.addNode( 8 ); // 8
	lg.addNode( 8 ); // 9
	lg.addEdge( 0, 1 );
	lg.addEdge( 0, 9 );
	lg.addEdge( 1, 2 );
	lg.addEdge( 1, 3 );
	lg.addEdge( 1, 4 );
	lg.addEdge( 1, 7 );
	lg.addEdge( 2, 3 );
	lg.addEdge( 3, 4 );
	lg.addEdge( 3, 5 );
	lg.addEdge( 4, 5 );
	lg.addEdge( 4, 8 );
	lg.addEdge( 5, 6 );
	lg.addEdge( 6, 7 );
	lg.addEdge( 7, 9 );
	
	std::cout	<<"\n--> the landscape filled : "<<std::endl;
	lg.printDOT(std::cout);
	std::cout	<<"\n"
				<<"\n--> energies : "<<std::endl;
	for (size_t i=0; i<lg.nodes(); i++) {
		std::cout <<" "<<i<<"("<<lg.getNodeEnergy(i)<<")";
	}
	std::cout <<std::endl;



		// start tests
	std::cout <<"->new S_Explicit mfe( lg, 2) = "; std::cout.flush();
	S_Explicit mfe(&lg,2);
	std::cout <<mfe.toString() <<std::endl;
	  // barrier tree to fill
	std::cout <<"->new LT_BarrierTree bt(-1.0) " <<std::endl;
	LT_BarrierTree bt(-1.0);
	std::cout <<"->bt.add(mfe) = "; std::cout.flush();
	std::cout <<bt.addMin(&mfe) <<std::endl;
	std::cout <<"\n===================================================\n";
	bt.write(std::cout, true);
	std::cout <<"\n===================================================\n";
	  // set up sampling object
	size_t upMaxLength = 3; 
	std::cout <<"->new WAC_MaxLength upWAC("<<upMaxLength<<")" <<std::endl;
	WAC_MaxLength upWAC(upMaxLength);
	std::cout <<"->new WalkRandom up(upWAC)" <<std::endl;
	WalkRandom up(&upWAC);
	std::cout <<"->new WalkAdaptive down(RANDOM)" <<std::endl;
	WalkAdaptive down(WalkAdaptive::RANDOM);
	bool addNewMin = true;
	std::cout <<"->new BS_UpDown barSampling(up,down,"<<(addNewMin?"true":"false")<<")" <<std::endl;
	BS_UpDown barSampling( &up, &down, addNewMin);
	  // set up sampling iteration stuff
	std::cout <<"->new MS_Uniform ms()" <<std::endl;
	MS_Uniform ms;
	size_t maxIterations = 2;
	std::cout <<"->new SAC_MaxIterations sac("<<maxIterations<<")" <<std::endl;
	SAC_MaxIterations sac(maxIterations);
	
	  // do sampling
	for (size_t i=0; i<8; i++) {
		sac.reset();
		std::cout <<"\n===================================================\n";
		std::cout <<"->barSampling.findBarriers(&bt, &ms, &sac) = ";std::cout.flush();
		std::cout <<barSampling.findBarriers(&bt, &ms, &sac) <<std::endl;
		
		bt.write(std::cout,  false);
		std::cout <<"\n===================================================\n";
	}
	std::cout <<"\n===================================================\n";
	bt.write(std::cout, true);
	std::cout <<"\n===================================================\n";
	
	
  	std::cout	<<"============= END OF TEST =================\n" <<std::endl;

	return 0;	
}
