#include "ell/Basin.hh"
#include "ell/S_Explicit.hh"
#include "biu/RandomNumberGenerator.hh"
#include "biu/RandomNumberFactory.hh"
#include <iostream>
#include <iomanip>



int main( int argc, char** argv ) {

	using namespace ell;
	

	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test function : Basin_FME\n"
				<<"===========================================" <<std::endl;

		// init randomizer	
	std::cout	<<"=> RND::setRNG( biu::RNG_LCG() )" <<std::endl;
	biu::RandomNumberGenerator* rng = new biu::RNG_LCG();
	biu::RNF::setRNG( rng );
	delete rng;
	const unsigned int seed = 1;
	std::cout	<<"=> biu::RNF::setSeed("<<seed<<")" <<std::endl;
	biu::RNF::getRNG().setSeed(seed);
	
	
	std::cout	<< "=> creating S_Explicit::LandscapeGraph lg" << std::endl;
	S_Explicit::LandscapeGraph lg;
	
	lg.addNode( 3 ); // 0
	lg.addNode( 5 ); // 1
	lg.addNode( 1 ); // 2
	lg.addNode( 4 ); // 3
	lg.addNode( 7 ); // 4
	lg.addNode( 6 ); // 5
	lg.addNode( 2 ); // 6
	lg.addNode( 7 ); // 7
	lg.addNode( 8 ); // 8
	lg.addNode( 8 ); // 9
	lg.addEdge( 0, 1 );
	lg.addEdge( 0, 9 );
	lg.addEdge( 1, 2 );
	lg.addEdge( 1, 4 );
	lg.addEdge( 1, 7 );
	lg.addEdge( 2, 3 );
	lg.addEdge( 3, 4 );
	lg.addEdge( 3, 5 );
	lg.addEdge( 4, 5 );
	lg.addEdge( 4, 8 );
	lg.addEdge( 5, 6 );
	lg.addEdge( 7, 9 );
	
	std::cout	<<"\n--> the landscape filled : "<<std::endl;
	lg.printDOT(std::cout);
	std::cout	<<"\n"
				<<"\n--> energies : "<<std::endl;
	for (size_t i=0; i<lg.nodes(); i++) {
		std::cout <<" "<<i<<"("<<lg.getNodeEnergy(i)<<")";
	}
	std::cout <<std::endl;


		// start tests
	std::cout <<"->new S_Explicit( lg, 2) = "; std::cout.flush();
	S_Explicit localMin(&lg,2);
	std::cout <<localMin.toString() <<std::endl;
	
	SC_Outstream scoBasin(std::cout, std::string(" <-- basin\n"));
	SPC_Outstream spcoSurface(std::cout, std::string(","), std::string(" <-- surface\n"));
	
	// check basin algorithm implementation
	
	std::cout <<"->flood( lm )" <<std::endl;
	double maxE = Basin_FME::flood(	&localMin,
								&scoBasin,
								99999.0,
								1.0,
								UINT_MAX,
								&spcoSurface
								);
	std::cout <<" max E flooded = "<<std::fixed <<std::setprecision(3) <<maxE <<std::endl;

	
	// check Wrapper class
	
	std::cout <<"->Basin_FME bas()" <<std::endl;
	Basin_FME bas(99999.0, 1.0, UINT_MAX);
	std::cout <<"->bas.enumerate( lm )" <<std::endl;
	maxE = bas.enumerate( &localMin, &scoBasin, &spcoSurface);
	std::cout <<" max E flooded = "<<std::fixed <<std::setprecision(3) <<maxE <<std::endl;
	
  	std::cout	<<"============= END OF TEST =================\n" <<std::endl;

	return 0;	
}
