#include "ell/LT_AutoFlood.hh"
#include "ell/LT_BarrierTree.hh"
#include "ell/S_Explicit.hh"
#include <iostream>
#include <iomanip>
#include <sstream>

using namespace ell;

void runFlooding(	const std::vector<S_Explicit>& minima
					, const double maxMergeBarrier
					, const double maxFloodHeight
					, std::ostream& out ) 
{

	std::cout <<"\n=========== maxMergeHeight = " <<maxMergeBarrier
				<<"  maxFloodHeight = " <<maxFloodHeight <<" ===========\n"
				<<std::endl;

	  // create barrier tree
	LT_BarrierTree bt(maxMergeBarrier);

	  // create flooding wrapper
	LT_AutoFlood lt( &bt, maxFloodHeight, 1.0, 999999);
	
	  // successively add all minima
	for (size_t i=0; i<minima.size(); i++) {
		if (lt.getMinIndex(&minima[i]) == LT_AutoFlood::INVALID_INDEX)
			lt.addMin(&minima[i]);
	}
	
	  // print resulting barrier tree
	lt.write( out, false);
}

//#include "ell/Walk.hh"
//void findGWE( const S_Explicit& s, std::ostream& out ) {
//	out <<"-->find GWE of " <<s.toString() <<std::endl;
//	SC_Outstream sc(out,"\n");
//	WalkGradient::walkGradient(&s, &sc);
//
//}


int main( int argc, char** argv ) {

	using namespace ell;
	

	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test function : LT_AutoFlood\n"
				<<"===========================================" <<std::endl;

	  // create energy landscape to test
	std::cout	<< "=> creating S_Explicit::LandscapeGraph lg" << std::endl;
	S_Explicit::LandscapeGraph lg;
	
	lg.addNode( 4 ); // 0
	lg.addNode( 3 ); // 1
	lg.addNode( 5 ); // 2
	lg.addNode( 4 ); // 3
	lg.addNode( 3 ); // 4
	lg.addNode( 2 ); // 5
	lg.addNode( 2 ); // 6
	lg.addNode( 1 ); // 7
	lg.addNode( 4 ); // 8
	lg.addNode( 6 ); // 9
	lg.addNode( 5 ); // 10
	lg.addNode( 5 ); // 11
	lg.addNode( 5 ); // 12
	
	lg.addEdge( 0, 1 );
	lg.addEdge( 1, 2 );
	lg.addEdge( 2, 3 );
	lg.addEdge( 3, 4 );
	lg.addEdge( 3, 5 );
	lg.addEdge( 3, 8 );
	lg.addEdge( 4, 5 );
	lg.addEdge( 4, 6 );
	lg.addEdge( 4, 7 );
	lg.addEdge( 6, 7 );
	lg.addEdge( 6, 8 );
	lg.addEdge( 7, 8 );
	lg.addEdge( 8, 10 );
	lg.addEdge( 8, 12 );
	lg.addEdge( 9, 11 );
	lg.addEdge( 9, 12 );
	lg.addEdge( 10, 12 );
	lg.addEdge( 11, 12 );
	
	  // create minima
	std::vector<S_Explicit> minima;
	minima.push_back(S_Explicit(&lg,1));
	minima.push_back(S_Explicit(&lg,7));
	minima.push_back(S_Explicit(&lg,5));
	minima.push_back(S_Explicit(&lg,11));

	std::cout	<<"\n--> the landscape filled : "<<std::endl;
	lg.printDOT(std::cout);
	std::cout	<<"\n"
				<<"\n--> energies : "<<std::endl;
	for (size_t i=0; i<lg.nodes(); i++) {
		std::cout <<" "<<i<<"("<<lg.getNodeEnergy(i)<<")";
	}
	std::cout <<std::endl;
	
//	for (size_t i=0; i<lg.nodes(); i++) {
//		findGWE( S_Explicit(&lg, i), std::cout );
//	}
	
	  // create a barrier tree where nothing is merged
	runFlooding( minima, -1.0, 0.0, std::cout);
	runFlooding( minima, -1.0, 1.0, std::cout);
	runFlooding( minima, -1.0, 2.0, std::cout);
	runFlooding( minima, -1.0, 10.0, std::cout);

	  // create a barrier tree where saddle points are merged
	runFlooding( minima, 0.0, 0.0, std::cout);
	runFlooding( minima, 0.0, 1.0, std::cout);
	runFlooding( minima, 0.0, 2.0, std::cout);

	  // create a barrier tree where minima are merged
	runFlooding( minima, 1.0, 0.0, std::cout);
	runFlooding( minima, 1.0, 1.0, std::cout);
	runFlooding( minima, 1.0, 2.0, std::cout);


	  // create a barrier tree where minima are merged
	runFlooding( minima, 2.0, 0.0, std::cout);
	runFlooding( minima, 2.0, 1.0, std::cout);
	runFlooding( minima, 2.0, 2.0, std::cout);
	
	  // check input output
	{
		std::stringstream ssBT;
		runFlooding( minima, 2.0, 2.0, ssBT);
		  // print resulting barrier tree
		LT_BarrierTree bt(2.0);
		bt.read( ssBT, &minima[0]);
		bt.write( std::cout, false);
	}

  	std::cout	<<"============= END OF TEST =================\n" <<std::endl;

	return 0;	
}
