
#include "ell/util/PriorityQueue.hh"
#include "ell/spinglass/S_SG_Ising.hh"
#include "biu/RandomNumberFactory.hh"
#include "ell/Basin.hh"

#include <iostream>


int main (int argc, char** argv) {

	using namespace ell;
	using namespace ell::util;
	
	typedef Basin_FME::QueueVal QV;

	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test function : PriorityQueue \n"
				<<"===========================================" <<std::endl;

	// init random number generator
	biu::RandomNumberGenerator* rng = new biu::RNG_LCG(1);
	biu::RNF::setRNG(rng);

	std::cout <<"->new PriorityQueue" <<std::endl;
	PriorityQueue<QV> pq;
	
	std::cout <<"->new S_SG_Ising sgi = "; std::cout.flush();
	S_SG_Ising sgi(5);
	std::cout <<sgi.toString() <<std::endl;
	
	size_t m = 5;
	std::cout <<"->adding " <<m <<" random neighbors" <<std::endl;
	for (size_t i=0; i<m; i++) {
		State * n = sgi.getRandomNeighbor();
		std::cout <<"  "<<i <<" : "<<n->toString() <<" " <<n->getEnergy(); std::cout.flush();
		PriorityQueue<QV>::InsertResult act = pq.insert(n);
		act.first->second.minNeighGWE = i;
		std::cout <<(act.second?" inserted":" replaced") <<std::endl;
		delete n;
	}
	
	
	std::cout <<"->pq->size() = "<<pq.size() <<std::endl;
	std::cout <<"->pq->getMaxE() = " <<pq.getMaxE() <<std::endl;
	std::cout <<"->pq->reduceMaxE(pq->getMaxE())" <<std::endl;
	pq.reduceMaxE(pq.getMaxE());
	std::cout <<"->pq->getMaxE() = " <<pq.getMaxE() <<std::endl;
	std::cout <<"->pq->size() = "<<pq.size() <<std::endl;
	
	std::cout <<"->PriorityQueue content:" <<std::endl;
	while(!pq.empty()) {
		PriorityQueue<QV>::const_iterator top = pq.top();
		State* next = sgi.uncompress(top->first.s,NULL);
		std::cout	<<"  ["<<top->first.E <<"," <<next->toString() <<"] = "
					<<" gwe=" <<top->second.minNeighGWE <<std::endl;
		if (next != NULL)
			delete next;
		pq.pop();
	}


  	std::cout	<<"============= END OF TEST =================\n" <<std::endl;

	return 0;
}
