

#include "ell/S_Explicit.hh"

#include <iostream>

#include <biu/RandomNumberFactory.hh>


using namespace ell;

int main(int argc, char** argv) {
	
	
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : S_Explicit\n"
				<<"===========================================" <<std::endl;


	std::cout	<< "=> creating S_Explicit::LandscapeGraph lg" << std::endl;
	S_Explicit::LandscapeGraph lg;
	
	lg.addNode( 3 );
	lg.addNode( 5 );
	lg.addNode( 1 );
	lg.addNode( 4 );
	lg.addNode( 7 );
	lg.addNode( 6 );
	lg.addNode( 2 );
	lg.addEdge( 0, 1 );
	lg.addEdge( 1, 2 );
	lg.addEdge( 1, 4 );
	lg.addEdge( 2, 3 );
	lg.addEdge( 3, 4 );
	lg.addEdge( 3, 5 );
	lg.addEdge( 5, 6 );
	
	std::cout	<<"\n--> the landscape filled : "<<std::endl;
	lg.printDOT(std::cout);
	std::cout	<<"\n"
				<<"\n--> energies : "<<std::endl;
	for (size_t i=0; i<lg.nodes(); i++) {
		std::cout <<" "<<i<<"("<<lg.getNodeEnergy(i)<<")";
	}
	std::cout <<std::endl;

	
	
	
	// init randomizer	
	std::cout	<<"=> RND::setRNG( biu::RNG_LCG() )" <<std::endl;
	biu::RandomNumberGenerator* rng = new biu::RNG_LCG();
	biu::RNF::setRNG( rng );
	delete rng;
	std::cout	<<"=> biu::RNF::setSeed(100)" <<std::endl;
	biu::RNF::getRNG().setSeed(100);
	
	
	std::cout	<< "=> creating S_Explicit s1( lg, 0 )" << std::endl;
	S_Explicit s1(&lg,3);
	std::cout	<< "=> s1.getID() = " <<s1.getID() << std::endl;
	
	std::cout	<< "=> creating S_Explicit s2( lg, 4 )" << std::endl;
	S_Explicit s2(&lg,4);
	
	
		// toString()
	std::cout	<<"->s1.toString()  = "
				<<s1.toString()
				<<std::endl
				<<"->s2.toString()  = "
				<<s2.toString()
				<<std::endl;
	
		// getEnergy()
	std::cout	<<"->s1.getEnergy()  = "
				<<(s1.getEnergy())
				<<std::endl;
	std::cout	<<"->s2.getEnergy()  = "
				<<(s2.getEnergy())
				<<std::endl;
	
	ell::State* sc1 = s1.clone();
	
		// clone()  ==
	std::cout	<<"->(*this == *(this->clone()))  = "
				<<(s1 == *sc1 ? "true":"false")
				<<std::endl;
	
		// clone()  !=
	std::cout	<<"->(*this != *(this->clone()))  = "
				<<(s1 != *sc1 ? "true":"false")
				<<std::endl;
		// clone(sc1)
	s2.clone(sc1);
	std::cout	<<"->(*x == *(x->clone(toFill)))  = "
				<<(s2 == *sc1 ? "true":"false")
				<<std::endl;
	
		// clone()  !=
	std::cout	<<"->(*x != *(x->clone(toFill)))  = "
				<<(s2 != *sc1 ? "true":"false")
				<<std::endl;
	delete sc1;
	
	s2.getRandomNeighbor(&s1);
	std::cout	<<"->s2.getRandomNeighbor(s1)"
				<<"\n  s2.toString()  = "
				<<s2.toString()
				<<"\n  s1.toString()  = "
				<<s1.toString()
				<<std::endl;
	
	
		// getMinimalDistance(..)
	std::cout	<<"Distance s1 - s2 : "
				<<s1.getMinimalDistance(s2)
				<<std::endl;
	
	int num = 0;
	
		// getNeighborList()
	ell::State::NeighborListPtr nl = s1.getNeighborList();
	std::set<std::string> neighs, randneighs;
	std::set<std::string>::const_iterator setIt;
	std::cout	<<"->s1.getNeighborList()  ==> now sorted" <<std::endl;
	for(ell::State::NeighborList::Iterator it = nl->begin();
		 it != nl->end(); ++it) {
		neighs.insert(it->toString());
	}
		std::cout <<" s1" <<"\t: " <<s1.toString() <<std::endl;
	for(setIt = neighs.begin(); setIt != neighs.end(); setIt++) {
		std::cout <<" " <<++num <<"\t: " <<*setIt <<std::endl;
	}
	
	
		// getRandomNeighborList()
	num = 0;
	nl = s1.getRandomNeighborList();
	std::cout	<<"->s1.getRandomNeighborList() ==> now sorted" <<std::endl;
	for(ell::State::NeighborList::Iterator it = nl->begin();
		 it != nl->end(); ++it) {
		randneighs.insert(it->toString());
	}
	for(setIt = randneighs.begin(); setIt != randneighs.end(); setIt++) {
		std::cout <<" " <<++num <<"\t: " <<*setIt <<std::endl;
	}
	
	std::cout	<<"S_Explicit(lg,2)<S_Explicit(lg,0) = "; std::cout.flush();
	std::cout	<<(S_Explicit(&lg,2)<S_Explicit(&lg,0)?"true":"false") 
				<<std::endl;

	

	return 0;

}
