#include "ell/rna/S_RNAfe_SingleM_TB.hh"
#include <time.h>
#include <iostream>
#include <fstream>
#include "biu/RandomNumberFactory.hh"

void testCompression(ell::State* s, ell::State* s2) {
	/*
	 * testing compress & uncompress functions.
	 */
	
	ell::State* s3 = s2->clone();
	
    /*
     * testing compress(void) & uncompress(const CSequence& cseq)
     */
    std::cout   << std::endl 
                << "testing compress(void) & uncompress(const CSequence& cseq):"
                << std::endl;
    std::cout   << "s->toString()  = " << s->toString() << std::endl;
    std::cout   << "cseq = s->compress()"  << std::endl;
    ell::CSequence cseq = s->compress();
    std::cout   << "cseq.size() = " << cseq.size() << std::endl;
    std::cout   << "s2->toString() = " << s2->toString() << std::endl;
    std::cout   << "s2->uncompress(cseq)" << std::endl;
    s2->uncompress(cseq);
    std::cout   << "s2->toString() = " << s2->toString() << std::endl;

    /*
     * testing  compress(CSequence& toFill) &
     *          uncompress(const CSequence& cseq, State* toFill)
     */
    std::cout   << std::endl 
                << "testing compress(CSequence& cseq) & "
                << "uncompress(const CSequence& cseq, State* toFill):"
                << std::endl;
    std::cout	<< "=> uncompress to new State" << std::endl;
    std::cout	<< "State* s4 = NULL" << std::endl;
    ell::State* s4 = NULL;
    std::cout   << "s4 = s->uncompress(cseq, s4);" << std::endl;
    s4 = s->uncompress(cseq, s4);
    std::cout   << "s4->toString() = " << s4->toString() << std::endl;
    std::cout	<< "=> uncompress to existing State" << std::endl;
    std::cout   << "s3->toString() = " << s3->toString() << std::endl;
    std::cout	<< "s3 = s->uncompress(cseq, s3)" << std::endl;
    s3 = dynamic_cast<ell::S_RNAfe_SingleM_TB*>(s->uncompress(cseq, s3));
    std::cout   << "s3->toString() = " << s3->toString() << std::endl;
    
    
    delete s3;
    delete s4;

}

void
testNeighborList() {
//	biu::Sequence rnaSeq = ell::RNAFreeEnergy_TB::getSequenceAlphabet()->getSequence("UGUCGUCGUAUGAUGCUGACUGUG");
//	biu::Structure rnaStr = ell::RNAFreeEnergy_TB::getStructureAlphabet()->getSequence(".((((.(((...)))).)))....");
	biu::Sequence rnaSeq = ell::RNAFreeEnergy_TB::getSequenceAlphabet()->getSequence("ACGCGUACGACACGCAACGCAGU");
	biu::Structure rnaStr = ell::RNAFreeEnergy_TB::getStructureAlphabet()->getSequence(".......................");
	  // create RNA molecule 
	ell::RNAFreeEnergy_TB * rna = new ell::RNAFreeEnergy_TB(&rnaSeq,&rnaStr,true);
	
	  // create start state for flooding
	ell::S_RNAfe_SingleM_TB * start = new ell::S_RNAfe_SingleM_TB(*rna);
	
	std::cout <<" neighbor list of " <<start->toString() <<std::endl;
	  // get neighbor list
	ell::State::NeighborListPtr neighbors = start->getNeighborList();
	ell::State::NeighborList::Iterator it = neighbors->begin();
		// contains all neighbors with E(top) < E < maxE
	  // iterate through all neighbors
	while (neighbors->end() != it) 
	{
		std::cout <<it->toString() <<std::endl;
		++it;
	}

	delete start;
	delete rna;

}

int main(int argc, char **argv) {

	using namespace ell;

	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : S_RNAfe_SingleM_TB\n"
				<<"===========================================" <<std::endl;
				
	 // create RNG and set for global use via biu::RNF 				
	biu::RandomNumberGenerator* rng = new biu::RNG_LCG();
	biu::RNF::setRNG(rng);	// set for global usage
	delete rng;
		// erzeugen	

		std::string seqStr = "GGCAAAGUCAAAGAU";
	std::cout	<<"=> " <<seqStr <<" = seqStr" <<std::endl;
		std::string structStr = "..(...).(...)..";
	std::cout	<<"=> " <<structStr <<" = structStr" <<std::endl;
	std::cout	<<"=> create ell::S_RNAfe_SingleM_TB(seqStr,"
				<<" structStr)" <<std::endl;
	ell::S_RNAfe_SingleM_TB rnaStruct(seqStr, structStr);
	std::cout	<< "=> rnaStruct.>getID() = " <<rnaStruct.getID() << std::endl;

		// toString()
	std::cout	<<"->toString()  = "
				<<rnaStruct.toString()
				<<std::endl;


	ell::State* rnaStruct2 = rnaStruct.clone();

		// clone()  ==
	std::cout	<<"->(*this == *(this->clone()))  = "
				<<(rnaStruct == *rnaStruct2 ? "true":"false")
				<<std::endl;

		// clone()  !=
	std::cout	<<"->(*this != *(this->clone()))  = "
				<<(rnaStruct != *rnaStruct2 ? "true":"false")
				<<std::endl;

	std::cout <<"r->clone(r2)" <<std::endl;
	rnaStruct.clone(rnaStruct2);
		// clone()  ==
	std::cout	<<"->(*r == *r2)  = "
				<<(rnaStruct == *rnaStruct2 ? "true":"false")
				<<std::endl;

		// clone()  !=
	std::cout	<<"->(*r != *r2)  = "
				<<(rnaStruct != *rnaStruct2 ? "true":"false")
				<<std::endl;

	delete rnaStruct2;
	
	int num = 0;
	
		// getNeighborList()
	ell::State::NeighborListPtr nl = rnaStruct.getNeighborList();
	std::set<std::string> neighs, randneighs;
	std::set<std::string>::const_iterator setIt;
	std::cout	<<"->getNeighborList()  ==> now sorted" <<std::endl;
	for(ell::State::NeighborList::Iterator it = nl->begin();
		 it != nl->end(); ++it) {
		neighs.insert(it->toString());
	}
	for(setIt = neighs.begin(); setIt != neighs.end(); setIt++) {
		std::cout <<" " <<++num <<"\t: " <<*setIt <<std::endl;
	}

		// getRandomNeighborList()
	num = 0;
	nl = rnaStruct.getRandomNeighborList();
	std::cout	<<"->getRandomNeighborList() ==> now sorted" <<std::endl;
	for(ell::State::NeighborList::Iterator it = nl->begin();
		 it != nl->end(); ++it) {
		randneighs.insert(it->toString());
	}
	for(setIt = randneighs.begin(); setIt != randneighs.end(); setIt++) {
		std::cout <<" " <<++num <<"\t: " <<*setIt <<std::endl;
	}

		// getMinimalDistance(..)
	std::cout	<<"Distance to GGCAAAGUCAAAGAU (.((...)....).): "
				<<rnaStruct.getMinimalDistance(
					ell::S_RNAfe_SingleM_TB(	"GGCAAAGUCAAAGAU", 
														"(.((...)....).)"))
				<<std::endl;

	ell::State * s2 = nl->begin()->clone();
	testCompression(&rnaStruct,s2);
	delete s2;
	
//	testNeighborList();
	
	// test fromString
	std::string strRep = ".((...).(...).)(GGCAAAGUCAAAGAU)";
	std::cout	<< "State* s_fromString = rnaStruct.fromString(\"" <<strRep<<"\")" << std::endl;
	State* s_fromString = rnaStruct.fromString(strRep);
	std::cout	<< "s_fromString->toString() = " <<s_fromString->toString() 
				<< std::endl;
	
	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	return 0;
}
