#include "ell/StateAcceptor.hh"
#include <iostream>

using namespace ell;

/*!
 * This is a dummy implementation of a State class. The only functionality
 * implemented is setting an id and energy in the constructor and retrieving it via
 * toString and getEnergy.
 * 
 * @author Daniel Maticzka
 */
class DummyState : public State
{
	public:
	
	std::string id;
	double energy;
	static const std::string ID;

	virtual
	const std::string& getID() const {
		return DummyState::ID;
	}

	DummyState(std::string _id, double _energy)
	{
		id = _id; 
		energy = _energy;
	}
	
	~DummyState()
	{}
	
	bool operator== (const State& state2) const {return true;}
	bool operator!= (const State& state2) const {return true;}

	double getEnergy() const 
	{
		return energy;
	}
	
	unsigned int getMinimalDistance(const State& state2) const 
	{
		return 0;
	}
	
	DummyState* clone(State* toFill = NULL) const { 
		if (toFill == NULL) return new DummyState(id, energy);
		DummyState* s = static_cast<DummyState*>(toFill);
		s->id = this->id;
		s->energy = this->energy;
		return s;
	}
	
	State* fromString(const std::string& stringRep) const { return NULL; }
	
	std::string	toString() const 
	{
		return id;
	}
	std::string	&toString( std::string& toFill) const {
		toFill = id;
		return toFill;
	}

	NeighborListPtr getNeighborList() const
	{
		return *(new NeighborListPtr);
	}

	NeighborListPtr getRandomNeighborList() const
	{
		return *(new NeighborListPtr);
	}

	State* getRandomNeighbor(State* inPlaceNeigh = NULL) const
	{
		return NULL;
	}

	//////////////  COMPRESSED REPRESENTATION  ////////////////////////////
	
	CSequence  compress(void) const {
		return CSequence();
	}
	
	CSequence&   compress(CSequence& toFill) const {
		toFill.clear();
		return toFill;
	}
	
	State*   uncompress(const CSequence& cseq, State* toFill) const {
		return toFill;
	}
	
	State*   uncompress(const CSequence& cseq) {
		return NULL;
	}

};
const std::string DummyState::ID = std::string("ell::test::DummyState");


bool
StateAcceptor::accept(const StateCollector* const sc, const State& succ) const
{
	return true;
}

int main(int argc, char** argv)
{
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : SA_All\n"
				<<"===========================================" <<std::endl;
	std::cout	<< "=> creating StateCollector" << std::endl;
	StateCollector* sc = new SC_Counting();
	std::cout	<< "=> creating State" << std::endl;
	State* s = new DummyState("State",0);
	std::cout	<< "=> creating SA_All" << std::endl;
	StateAcceptor* sa = new SA_All();
	
	std::cout	<< "->accept = " << sa->accept(sc,*s) << std::endl;
	
	delete sa;
	delete sc;
	delete s;
	std::cout	<<"============= END OF TEST =================\n" <<std::endl;

	
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : SA_E_Gr\n"
				<<"===========================================" <<std::endl;
	std::cout	<< "=> creating DummyState(\"scstate\", 5)" << std::endl;
	State* scstate = new DummyState("scstate", 5);
	std::cout	<< "=> creating StateCollector" << std::endl;
	sc = new SC_Counting();
	std::cout	<< "->add(scstate)" << std::endl;
	sc->add(*scstate);
	
	std::cout	<< "=> creating DummyState(\"State\", 4)" << std::endl;
	s = new DummyState("State", 4);
	std::cout	<< "=> creating SA_E_Gr" << std::endl;
	sa = new SA_E_Gr();
	std::cout	<< "->accept = " << sa->accept(sc,*s) << std::endl;
	delete sa;
	delete s;
	
	std::cout	<< "=> creating DummyState(\"State\", 6)" << std::endl;
	s = new DummyState("State", 6);
	std::cout	<< "=> creating SA_E_Gr" << std::endl;
	sa = new SA_E_Gr();
	std::cout	<< "->accept = " << sa->accept(sc,*s) << std::endl;
	delete sa;
	delete s;

	delete sc;
	delete scstate;
	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : SA_E_Le\n"
				<<"===========================================" <<std::endl;
	std::cout	<< "=> creating DummyState(\"scstate\", 5)" << std::endl;
	scstate = new DummyState("scstate", 5);
	std::cout	<< "=> creating StateCollector" << std::endl;
	sc = new SC_Counting();
	std::cout	<< "->add(scstate)" << std::endl;
	sc->add(*scstate);
	
	std::cout	<< "=> creating DummyState(\"State\", 4)" << std::endl;
	s = new DummyState("State", 4);
	std::cout	<< "=> creating SA_E_Le" << std::endl;
	sa = new SA_E_Le();
	std::cout	<< "->accept = " << sa->accept(sc,*s) << std::endl;
	delete sa;
	delete s;
	
	std::cout	<< "=> creating DummyState(\"State\", 6)" << std::endl;
	s = new DummyState("State", 6);
	std::cout	<< "=> creating SA_E_Le" << std::endl;
	sa = new SA_E_Le();
	std::cout	<< "->accept = " << sa->accept(sc,*s) << std::endl;
	delete sa;
	delete s;

	delete sc;
	delete scstate;
	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : SA_E_Eq\n"
				<<"===========================================" <<std::endl;
	std::cout	<< "=> creating DummyState(\"scstate\", 5)" << std::endl;
	scstate = new DummyState("scstate", 5);
	std::cout	<< "=> creating StateCollector" << std::endl;
	sc = new SC_Counting();
	std::cout	<< "->add(scstate)" << std::endl;
	sc->add(*scstate);
	
	std::cout	<< "=> creating DummyState(\"State\", 4)" << std::endl;
	s = new DummyState("State", 4);
	std::cout	<< "=> creating SA_E_Eq" << std::endl;
	sa = new SA_E_Eq();
	std::cout	<< "->accept = " << sa->accept(sc,*s) << std::endl;
	delete sa;
	delete s;
	
	std::cout	<< "=> creating DummyState(\"State\", 5)" << std::endl;
	s = new DummyState("State", 5);
	std::cout	<< "=> creating SA_E_Eq" << std::endl;
	sa = new SA_E_Eq();
	std::cout	<< "->accept = " << sa->accept(sc,*s) << std::endl;
	delete sa;
	delete s;

	delete sc;
	delete scstate;
	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : SA_E_GrEq\n"
				<<"===========================================" <<std::endl;
	std::cout	<< "=> creating DummyState(\"scstate\", 5)" << std::endl;
	scstate = new DummyState("scstate", 5);
	std::cout	<< "=> creating StateCollector" << std::endl;
	sc = new SC_Counting();
	std::cout	<< "->add(scstate)" << std::endl;
	sc->add(*scstate);
	
	std::cout	<< "=> creating DummyState(\"State\", 4)" << std::endl;
	s = new DummyState("State", 4);
	std::cout	<< "=> creating SA_E_GrEq" << std::endl;
	sa = new SA_E_GrEq();
	std::cout	<< "->accept = " << sa->accept(sc,*s) << std::endl;
	delete sa;
	delete s;
	
	std::cout	<< "=> creating DummyState(\"State\", 5)" << std::endl;
	s = new DummyState("State", 5);
	std::cout	<< "=> creating SA_E_GrEq" << std::endl;
	sa = new SA_E_GrEq();
	std::cout	<< "->accept = " << sa->accept(sc,*s) << std::endl;
	delete sa;
	delete s;

	delete sc;
	delete scstate;
	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : SA_E_LeEq\n"
				<<"===========================================" <<std::endl;
	std::cout	<< "=> creating DummyState(\"scstate\", 5)" << std::endl;
	scstate = new DummyState("scstate", 5);
	std::cout	<< "=> creating StateCollector" << std::endl;
	sc = new SC_Counting();
	std::cout	<< "->add(scstate)" << std::endl;
	sc->add(*scstate);
	
	std::cout	<< "=> creating DummyState(\"State\", 6)" << std::endl;
	s = new DummyState("State", 6);
	std::cout	<< "=> creating SA_E_LeEq" << std::endl;
	sa = new SA_E_LeEq();
	std::cout	<< "->accept = " << sa->accept(sc,*s) << std::endl;
	delete sa;
	delete s;
	
	std::cout	<< "=> creating DummyState(\"State\", 5)" << std::endl;
	s = new DummyState("State", 5);
	std::cout	<< "=> creating SA_E_LeEq" << std::endl;
	sa = new SA_E_LeEq();
	std::cout	<< "->accept = " << sa->accept(sc,*s) << std::endl;
	delete sa;
	delete s;

	delete sc;
	delete scstate;
	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	return 0;
}
