#include "ell/StateCollector.hh"
#include "ell/State.hh"
#include <iostream>
#include <iomanip>
#include <iterator>
#include <vector>
#include <list>


#include "ell/LT_MinimaSet.hh"

#include "DummyState.hh"

using namespace ell;



void fillSC( StateCollector& sc, const std::list<State*>& l ) {
	std::list<State*>::const_iterator it = l.begin();
	std::cout << "->add( States )" << std::endl;
	for (; it != l.end(); it++)
		sc.add(**it);
}

void printSCstat(const StateCollector& sc, std::ostream &out) {
	
	out	<<"->getlastAdded()->toString() = " 
		<<sc.getLastAdded()->toString()
		<<std::endl; 
	out	<<"->size() = " 
		<<sc.size()
		<<std::endl; 
}

int main(int argc, char** argv)
{
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test interface : StateCollector\n"
				<<"===========================================" <<std::endl;
	
	// create some States
	std::cout 	<< "=> creating 5 DummyStates";
	std::list<State*> l;
	l.push_back(new DummyState("State 2",2.0, 20));
	l.push_back(new DummyState("State 3",3.0, 20));
	l.push_back(new DummyState("State 4",4.0, 20));
	l.push_back(new DummyState("State 5",5.0, 20));
	std::list<State*>::const_iterator it;
	
	
	{
		std::cout	<<std::endl
					<<"===========================================\n"
					<<" test class : SC_Counting\n"
					<<"===========================================" <<std::endl;
		  // create new object	
		std::cout 	<< "=> creating ell::SC_Counting" << std::endl;
		SC_Counting sc = SC_Counting();
		  // fill SC
		fillSC(sc, l);
		  // get SC stats
		printSCstat(sc,std::cout);
	}

	{
		std::cout	<<std::endl
					<<"===========================================\n"
					<<" test class : SC_Outstream\n"
					<<"===========================================" <<std::endl;
		  // create new object	
		std::cout 	<< "=> creating ell::SC_Outstream" << std::endl;
		SC_Outstream sc = SC_Outstream(std::cout, " accepted\n");
		  // fill SC
		fillSC(sc, l);
		  // get SC stats
		printSCstat(sc,std::cout);
	}

	{
		std::cout	<<std::endl
					<<"===========================================\n"
					<<" test class : SC_Listing\n"
					<<"===========================================" <<std::endl;
		  // create new object	
		std::cout 	<< "=> creating ell::SC_Listing" << std::endl;
		SC_Listing sc = SC_Listing();
		  // fill SC
		fillSC(sc, l);
		  // get SC stats
		printSCstat(sc,std::cout);
		std::cout <<"=>stored list :" <<std::endl;
		SC_Listing::List sl = sc.getList();
		for( SC_Listing::List::const_iterator x = sl.begin(); x!=sl.end(); x++)
			std::cout <<"  " <<(*x)->toString() <<std::endl;
	}

	{
		std::cout	<<std::endl
					<<"===========================================\n"
					<<" test class : SC_ListingK\n"
					<<"===========================================" <<std::endl;
		  // create new object	
		const size_t k = 3;
		std::cout 	<< "=> creating ell::SC_ListingK k = "<<k << std::endl;
		SC_ListingK sc = SC_ListingK(k);
		  // fill SC
		fillSC(sc, l);
		fillSC(sc, l);
		  // get SC stats
		printSCstat(sc,std::cout);
		std::cout <<"=>stored list :" <<std::endl;
		SC_ListingK::iterator sl = sc.getList();
		do {
			std::cout <<"  " <<(*sl)->toString() <<std::endl;
		} while (++sl);
	}

	{
		std::cout	<<std::endl
					<<"===========================================\n"
					<<" test class : SC_ListingCompr\n"
					<<"===========================================" <<std::endl;
		  // create new object	
		std::cout 	<< "=> creating ell::SC_ListingCompr" << std::endl;
		SC_ListingCompr sc = SC_ListingCompr();
		  // fill SC
		fillSC(sc, l);
		  // get SC stats
		printSCstat(sc,std::cout);
		std::cout <<"=>stored uncompressed list :" <<std::endl;
		SC_ListingCompr::List sl = sc.getList();
		for( SC_ListingCompr::List::const_iterator x = sl.begin(); x!=sl.end(); x++) {
			std::cout <<"  " << DummyState::a.getString(DummyState::a.decompress(*x,7)) <<std::endl;
		}
	}

	{
		std::cout	<<std::endl
					<<"===========================================\n"
					<<" test class : SC_Maximum\n"
					<<"===========================================" <<std::endl;
		  // create new object	
		std::cout 	<< "=> creating ell::SC_Maximum" << std::endl;
		SC_Maximum sc = SC_Maximum();
		  // fill SC
		fillSC(sc, l);
		  // get SC stats
		printSCstat(sc,std::cout);
		std::cout	<<"=>maximal Energy State : " 
					<< sc.getMaximumState()->toString()
					<<std::endl;
	}

	{
		std::cout	<<std::endl
					<<"===========================================\n"
					<<" test class : SC_PartitionFunction\n"
					<<"===========================================" <<std::endl;
		  // create new object	
		const double kt_tmp =  LandscapeTopology::BOLTZMANN_KT;
		std::cout 	<< "=> set LandscapeTopology::BOLTZMANN_KT = 1.0" 
					<< std::endl;
		LandscapeTopology::BOLTZMANN_KT = 1.0;
		
		std::cout 	<< "=> creating ell::SC_PartitionFunction" << std::endl;
		SC_PartitionFunction sc = SC_PartitionFunction();
		  // fill SC
		fillSC(sc, l);
		  // get SC stats
		printSCstat(sc,std::cout);
		std::cout	<<"=>partition function : " 
//					<<std::fixed <<std::setprecision(7) 
					<< sc.getZ()
					<<std::endl;
		std::cout	<<"=>energy : " 
//					<<std::fixed <<std::setprecision(7) 
					<< sc.getEnergy()
					<<std::endl;
		std::cout 	<< "=> undo LandscapeTopology::BOLTZMANN_KT change" 
					<<std::endl;
		LandscapeTopology::BOLTZMANN_KT = kt_tmp;
	}


	{
		std::cout	<<std::endl
					<<"===========================================\n"
					<<" test class : SC_PartitionFunction_Shifted\n"
					<<"===========================================" <<std::endl;
		  // create new object	
		const double kt_tmp =  LandscapeTopology::BOLTZMANN_KT;
		std::cout 	<< "=> set LandscapeTopology::BOLTZMANN_KT = 1.0" 
					<< std::endl;
		LandscapeTopology::BOLTZMANN_KT = 1.0;
		
		
		std::cout 	<< "=> creating ell::SC_PartitionFunction_Shifted" 
					<< std::endl;
		SC_PartitionFunction_Shifted sc = SC_PartitionFunction_Shifted(0.0);
		  // fill SC
		fillSC(sc, l);
		  // get SC stats
		printSCstat(sc,std::cout);
		std::cout	<<"=>partition function : " 
//					<<std::fixed <<std::setprecision(7) 
					<< sc.getZ()
					<<std::endl;
		std::cout	<<"=>energy : " 
//					<<std::fixed <<std::setprecision(7) 
					<< sc.getEnergy()
					<<std::endl;
		std::cout 	<< "=> undo LandscapeTopology::BOLTZMANN_KT change" 
					<<std::endl;
		LandscapeTopology::BOLTZMANN_KT = kt_tmp;
	}

	  // clear memory of dummy states
	for (it = l.begin(); it != l.end(); it++)
		delete (*it);

	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	return 0;
}
