#include "ell/Walk.hh"
#include "ell/NeighborGenerator.hh"
#include "ell/StateAcceptor.hh"
#include "ell/WalkAbortionCriterion.hh"
#include "ell/State.hh"
#include <iostream>
#include <string>
#include <vector>

#include "DummyState.hh"

using namespace ell;

int main(int argc, char **argv) {
	
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : Walk\n"
				<<"===========================================" <<std::endl;

	std::cout	<< "=> creating DummyState" << std::endl;
	State* state = new DummyState("state 1", 10, 20);
	std::cout	<< "->getEnergy = " << state->getEnergy() << std::endl;
	
	std::cout	<< "=> creating NeighborGenerator" << std::endl;
	NeighborGenerator* ng = new NG_Successive();
	std::cout	<< "=> creating StateAcceptor" << std::endl;
	StateAcceptor* sa = new SA_E_Le();
	std::cout	<< "=> creating WalkAbortionCriterion" << std::endl;
	WalkAbortionCriterion* wac = new WAC_MaxLength(5);
	std::cout	<< "=> creating StateCollector scAccepted" << std::endl;
	StateCollector* scAccepted = new SC_Listing();
	std::cout	<< "=> creating StateCollector scChecked" << std::endl;
	StateCollector* scChecked = new SC_Listing();
	scChecked->add(*state);
	scChecked->add(*state);
	scChecked->add(*state);
				
	std::cout	<< "=> calling walk" << std::endl;
	scAccepted = Walk::walk(state, scAccepted, ng, sa, wac, scChecked);
	
	std::cout 	<< "Number Of Accepted States: " 
					<< scAccepted->size()
					<< std::endl;
	
	SC_Listing* sch = dynamic_cast<SC_Listing*>(scAccepted);
	
	std::cout 	<< "Accepted States:" << std::endl;
	SC_Listing::List list = sch->getList();
	SC_Listing::List::iterator iter;
  	for( iter = list.begin(); iter != list.end(); iter++ ) {
    	std::cout 	<< "  * Energy = "
    				<< (*iter)->getEnergy() << std::endl;
  	}
  	
	sch = dynamic_cast<SC_Listing*>(scChecked);
	
	std::cout 	<< "Checked States:" << std::endl;
	list = sch->getList();
  	for( iter = list.begin(); iter != list.end(); iter++ ) {
    	std::cout 	<< "  * Energy = "
    				<< (*iter)->getEnergy() << std::endl;
  	}
	
  	delete state;
  	delete ng;
  	delete sa;
  	delete wac;
  	delete scAccepted;
  	delete scChecked;
  	
  	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
  	
	return 0; 
}
