#include "ell/Walk.hh"
#include "ell/S_Explicit.hh"
#include "ell/StateCollector.hh"
#include "ell/StateAcceptor.hh"
#include <iostream>
#include <string>
#include <vector>
#include <stdlib.h>
#include <math.h>
#include "biu/RandomNumberGenerator.hh"
#include "biu/RandomNumberFactory.hh"


using namespace ell;



int main(int argc, char **argv) {
	
	
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : walkMC\n"
				<<"===========================================" <<std::endl;
	

	std::cout	<< "=> creating S_Explicit::LandscapeGraph lg" << std::endl;
	S_Explicit::LandscapeGraph lg;
	
	lg.addNode( 3 );
	lg.addNode( 6 );
	lg.addNode( 1 );
	lg.addNode( 4 );
	lg.addNode( 7 );
	lg.addNode( 5 );
	lg.addNode( 2 );
	lg.addEdge( 0, 1 );
	lg.addEdge( 1, 2 );
	lg.addEdge( 1, 4 );
	lg.addEdge( 2, 3 );
	lg.addEdge( 3, 4 );
	lg.addEdge( 3, 5 );
	lg.addEdge( 5, 6 );
	
	std::cout	<<"\n--> the landscape filled : "<<std::endl;
	lg.printDOT(std::cout);
	std::cout	<<"\n"
				<<"\n--> energies : "<<std::endl;
	for (size_t i=0; i<lg.nodes(); i++) {
		std::cout <<" "<<i<<"("<<lg.getNodeEnergy(i)<<")";
	}
	std::cout <<std::endl;

	

		// init randomizer	
	std::cout	<<"=> RND::setRNG( biu::RNG_LCG() )" <<std::endl;
	biu::RandomNumberGenerator* rng = new biu::RNG_LCG();
	biu::RNF::setRNG( rng );
	delete rng;
	const unsigned int seed = 1;
	std::cout	<<"=> biu::RNF::setSeed("<<seed<<")" <<std::endl;
	biu::RNF::getRNG().setSeed(seed);
	
	std::cout	<< "==> testing RNG" << std::endl;
	for (int i=1; i<=10; i++){
		std::cout 	<< i << ": " << (((double) biu::RNF::getRN()) / ((double) biu::RNF::getMaxRN())) << std::endl;
	}
	
	const double kT = 1.0;
	std::cout	<< "=> evaluation of used metropolis criterion:" << std::endl;
	for (int E=-7; E<8; E++) {
		std::cout	<< "exp((-1)* "<<E<<" / " <<kT <<" ) = " 
					<< exp((-1.0)* ((double)E) /kT ) 
					<< std::endl;
	}


	
	SC_Listing scl;
	std::cout	<< "=> creating S_Explicit s3" << std::endl;
	scl.add(ell::S_Explicit(&lg,3));
	std::cout	<< "=> creating S_Explicit s5" << std::endl;
	ell::S_Explicit s5(&lg,5);
	std::cout	<< " s5->getEnergy = " << s5.getEnergy() << std::endl;
	std::cout	<< "=> Neighbors of s5:" << std::endl;
	State::NeighborListPtr neighbors = s5.getNeighborList();
	State::NeighborList::Iterator it = neighbors->begin();
	size_t i=0;
	while( it != neighbors->end()) {
		std::cout	<< ++i << ". successor: " <<it->toString() <<" : "<< it->getEnergy() << std::endl;
		++it;
	}
	
	std::cout	<<"=> biu::RNF::setSeed("<<seed<<")" <<std::endl;
	biu::RNF::getRNG().setSeed(seed);
	std::cout	<< "=> creating SA_Metropolis sam with kT = "<<kT << std::endl;
	SA_Metropolis sam(kT);
	for (int i=1; i<=10; i++){
		std::cout 	<<"sam.accept(old energy: "<<scl.getLastAdded()->getEnergy()
					<<", successor energy: "<<s5.getEnergy()<<"): " 
					<<(sam.accept(&scl, s5)?"accepted":"rejected") 
					<<std::endl;
	}
	
	std::cout	<<"=> biu::RNF::setSeed("<<seed<<")" <<std::endl;
	biu::RNF::getRNG().setSeed(seed);
	const int maxstates = 10;
	std::cout	<< "=> calling walk ("<<scl.getLastAdded()->toString() <<", scList, wacMaxL(" <<maxstates <<"), kT="<<kT <<")" << std::endl;
	ell::WAC_MaxLength wac(maxstates);
	WalkMC::walkMC(scl.getLastAdded(), &scl, &wac, kT);
	
	std::cout 	<< "Number Of Accepted States: " 
					<< scl.size()
					<< std::endl;
	
	
	std::cout 	<< "Accepted States:" << std::endl;
	SC_Listing::List list = scl.getList();
	SC_Listing::List::iterator iter;
  	for( iter = list.begin(); iter != list.end(); iter++ ) {
    	std::cout 	<< "  " <<(*iter)->toString()
    				<< "  Energy = "
    				<< (*iter)->getEnergy() << std::endl;
  	}

  	
  	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
  	
	return 0; 
}
