#include "ell/Walk.hh"
#include "ell/State.hh"
#include <iostream>
#include <string>
#include <vector>
#include "biu/RandomNumberGenerator.hh"
#include "biu/RandomNumberFactory.hh"

#include "DummyState.hh"

using namespace ell;


int main(int argc, char **argv) {
	
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : walkRandom\n"
				<<"===========================================" <<std::endl;

		// init randomizer	
	std::cout	<<"=> RND::setRNG( biu::RNG_LCG() )" <<std::endl;
	biu::RandomNumberGenerator* rng = new biu::RNG_LCG();
	biu::RNF::setRNG( rng );
	delete rng;
	const unsigned int seed = 1;
	std::cout	<<"=> biu::RNF::setSeed("<<seed<<")" <<std::endl;
	biu::RNF::getRNG().setSeed(seed);
	
	std::cout	<< "=> creating DummyState" << std::endl;
	State* state = new DummyState("state 1", 10, 20);
	std::cout	<< "->getEnergy = " << state->getEnergy() << std::endl;

	// stop after 5 successors
	unsigned int l = 5;
	std::cout	<< "=> creating WAC_MaxLength("<<l<<")" << std::endl;
	WalkAbortionCriterion* wac = new WAC_MaxLength(l);
	std::cout	<< "=> creating StateCollector" << std::endl;
	StateCollector* sc = new SC_Listing();
	
	std::cout	<< "=> calling walk" << std::endl;
	sc = WalkRandom::walkRandom(state, sc, wac);
	
	std::cout 	<< "Number Of Accepted States (should be 6): " 
					<< sc->size()
					<< std::endl;
	
	SC_Listing* sch;
	sch = dynamic_cast<SC_Listing*>(sc);
	
	std::cout 	<< "Accepted States:" << std::endl;
	SC_Listing::List list = sch->getList();
	SC_Listing::List::iterator iter;
  	for( iter = list.begin(); iter != list.end(); iter++ ) {
    	std::cout 	<< "  * Energy = "
    				<< (*iter)->getEnergy() << std::endl;
  	}
	
  	delete state;
  	delete wac;
  	delete sc;
  	
  	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
  	
	return 0; 
}
